using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;

namespace SharpProxyService
{
    public class Config
    {
        string FileName;
        public bool Verbose = false;
        public Config(string CWD) 
        {
            FileName = CWD + "\\mod_rewrite.ini";
            Rules = new List<Rule>();
        }

        public void LoadOrSetDefaults() 
        {
                
                if (File.Exists(FileName))
                {
                    lock (Rules) 
                    {
                        Rules = new List<Rule>();
                        try
                        {
                            string[] RR = File.ReadAllLines(FileName);
                            foreach (string S in RR)
                            {
                                Rule R = new Rule();
                                R.FromString(S);
                                Rules.Add(R);
                            }
                        }
                        catch { }
                    }

                }
                else 
                {
                    Save();
                }


            
            
        
        }

        public void Save() 
        {
            
            lock (Rules) 
            {
                List<string> RR = new List<string>();

                foreach (Rule R in Rules) 
                {
                    RR.Add(R.ToString());
                }

                try
                {
                    File.WriteAllLines(FileName, RR.ToArray());
                }
                catch { }
            }
        }

        public List<Rule> Rules;
        
    }

    public class Rule 
    {
        public string URL_LIKE_MASK="";
        public bool Change_URL = false;
        public string URL_Change_Pattern="";
        public bool Change_UA = false;
        public string UA_Change_Pattern="";
        public bool Change_REFERER = false;
        public string REFERER_Change_Pattern="";
        public Rule() { }
        string Delim = "\t";
        public override string ToString()
        {
            return URL_LIKE_MASK + Delim + Change_URL.ToString() + Delim + URL_Change_Pattern +Delim+ Change_UA.ToString() +Delim+ UA_Change_Pattern+Delim+Change_REFERER.ToString() +Delim+ REFERER_Change_Pattern   ;
        }
        public void FromString(string S) 
        {
            try
            {
                string[] SPLIT = S.Split(Convert.ToChar(Delim));
                URL_LIKE_MASK = SPLIT[0];
                Change_URL = bool.Parse(SPLIT[1]);
                URL_Change_Pattern = SPLIT[2];
                Change_UA = bool.Parse(SPLIT[3]);
                UA_Change_Pattern = SPLIT[4];
                Change_REFERER = bool.Parse(SPLIT[5]);
                REFERER_Change_Pattern = SPLIT[6];
            }
            catch (Exception E) 
            {
                //System.Windows.Forms.MessageBox.Show(E.ToString());
            }
        }

        public bool  Apply(ProxyRequest PR) 
        {
            Regex R = new Regex(URL_LIKE_MASK);
            if (R.IsMatch(PR.URL)) 
            {
                if (Change_URL)
                {
                    string NewURL = R.Replace(PR.URL, URL_Change_Pattern);
                    char[] C = new char[1];
                    C[0] = Convert.ToChar("/");
                    string[] SPLIT =  NewURL.Split(C, 4);
                    PR.FileName = "/" + SPLIT[3];
                }
                if (Change_UA) 
                {
                    if (PR.HeadersFields.ContainsKey("USER-AGENT:"))
                    {
                        PR.HeadersFields["USER-AGENT:"]= UA_Change_Pattern;
                    }
                    else 
                    {
                        PR.HeadersFields.Add("USER-AGENT:", UA_Change_Pattern);
                    }
                }
                if (Change_REFERER) 
                {
                    string NEW_REFERER = R.Replace(PR.URL, REFERER_Change_Pattern );
                    if (PR.HeadersFields.ContainsKey("REFERER:"))
                    {
                        PR.HeadersFields["REFERER:"] = NEW_REFERER;
                    }
                    else
                    {
                        PR.HeadersFields.Add("REFERER:", NEW_REFERER);
                    }
                
                }
                return true;
            }
            return false;
        }

    }
}
